use Test::More tests => 5;
use strict;
use utf8;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Customer::API::Shop qw ( GetCustomerFolder GetNextCustomerNo );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObjectByPath LoadRootObject );
use DE_EPAGES::Order::API::Order qw ( CreateOrderWithContainer );

#========================================================================================
# EPG-25798: Can't add a product to the basket if the name of the default payment method
#            is longer than 255 characters.
#========================================================================================
sub TestPaymentMethodName {
    my $System = LoadRootObject();
    my $Shop = $System->child('TestShop');

    # (1) Add a new payment method having a too long name (length > 255)
    $Shop->featureMaxValue('PaymentMethods', 1);
    my $PaymentTypes = LoadObjectByPath('/PaymentTypes');
    my $Name = 'Paiement par carte de crédit via Le Crédit Lyonnais  Paiement 100% ' .
               'sécurisé (toutes nos transactions sont sécurisées par le service ' .
               'CHERLOCK paiement de la banque Le Crédit Lyonnais, les numéros de ' .
               'cartes bancaires ne circulent jamais en clair sru le net et ne sont ' .
               'en aucun cas communiqès(technologie de cryptographie)';
    my $PaymentMethod = $PaymentTypes->child('Free')->newPaymentMethod($Shop, {
        Alias       => 'TestPaymentMethod',
        IsActivated => 1,
        IsDefault   => 1,
    });
    my $LanguageID = $Shop->get('LanguageID');
    $PaymentMethod->set({ Name => $Name }, $LanguageID);
    is( $PaymentMethod->get('Name', $LanguageID), $Name, 'payment name set' );

    # (2) Create an order, and add a product to the line-item container
    my $Customer = LoadClassByAlias('Customer')->insertObject({
        Parent => GetCustomerFolder($Shop),
        Alias  => GetNextCustomerNo($Shop),
    });
    my $Order = CreateOrderWithContainer($Customer);
    my $Container = $Order->get('LineItemContainer');

    $Shop->featureMaxValue('Products', 1);
    my $ProductFolder = $Shop->folder('Products');
    my $Product = $ProductFolder->insertChild('Product', {
        Alias => 'TestProduct',
        Shop  => $Shop,
    });
    $Product->setListPrice($Shop->get('CurrencyID'), 0, 1);
    $Product->addToContainer($Container, { Quantity => 1 });

    is( $PaymentMethod->get('Name', $LanguageID), $Name,
        'payment name still unchanged' );
    my $ShortenedName = substr ($Name, 0, 252) . '...'; ## no critic qw(ProhibitMagicNumbers)
    is ($Container->get('Payment')->get('Name', $LanguageID), $ShortenedName,
        'payment name shortened to be storable in the database' );
}

sub Cleanup {
    my $System = LoadRootObject();
    $System->child('TestShop')->delete if ($System->existsChild('TestShop'));

    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Shop', { Alias => 'TestShop' });

    return 1;
}

RunScript(
    Message => 'long',
    Sub => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub   => sub {
                ok( Init(), 'initialization done' );

                TestPaymentMethodName();

                ok( Cleanup(), 'cleanup done' );
            }
        );
    },
);

1;