use Test::More tests => 6;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsClassByAlias
    LoadClassByAlias
    LoadRootObject
);
use DE_EPAGES::Permission::API::Constants qw ( $ATTRIBUTE_PERMISSION_READ );
use DE_EPAGES::Permission::API::Permission qw ( Everyone );
use DE_EPAGES::Permission::API::Table::AttributePermission qw (
    DeleteAttributePermission
    ExistsAttributePermission
    InsertAttributePermission
);

use Readonly;
Readonly::Scalar my $CLASSNAME => 'FancyClass';

sub Main {
    ok( Init(), 'initialization' );

    my $System = LoadRootObject();
    my $GroupsFolder = $System->child('Groups');

    my $ChildGroup = $GroupsFolder->child('TestGroupX');
    die 'GroupX does not exist' unless defined $ChildGroup;
    my $ParentGroup = $GroupsFolder->child('TestGroupY');
    die 'GroupY does not exist' unless defined $ParentGroup;
    $ParentGroup->addMember($ChildGroup);

    my $ObjectClass = LoadClassByAlias('Object');
    my $Class = $ObjectClass->insertSubClass({ 'Alias' => $CLASSNAME });
    my $Attribute = $Class->insertAttribute({
        'Alias'   => 'TestAttribute',
        'Type'    => 'String',
        'Package' => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute',
    });

    TestGroupAttributePermission($Attribute, $ChildGroup, $ParentGroup);

    ok( Cleanup(), 'cleanup done' );

    return;
}

sub TestGroupAttributePermission {
    my ($Attribute, $ChildGroup, $ParentGroup) = @_;

    my $NumEveryonePermissions = Everyone()->attributePermissions;

    my $hAttributePermission = {
        'AttributeID' => $Attribute->id,
        'TrusteeID'   => $ParentGroup->id,
        'Permission'  => $ATTRIBUTE_PERMISSION_READ,
    };
    InsertAttributePermission($hAttributePermission);
    ok( ExistsAttributePermission($hAttributePermission),
        'read permissions for the attribute were inserted' );

    my $ahAttributePermissions = $ChildGroup->directAttributePermissions;
    is( scalar @{ $ahAttributePermissions }, 0,
        'no attribute permissions defined on group ' . $ChildGroup->alias );

    $ahAttributePermissions = $ChildGroup->attributePermissions;
    is( scalar @{ $ahAttributePermissions }, (scalar @{ $NumEveryonePermissions })+1,
        'one attribute permission defined in group ' . $ChildGroup->alias .
        ', which is inherited by ' . $ParentGroup->alias
    );

    # cleanup
    DeleteAttributePermission($hAttributePermission);
    ok( !ExistsAttributePermission($hAttributePermission),
        'read permissions for the attribute were deleted' );

    return;
}

sub Init {
    Cleanup();

    my $System = LoadRootObject();
    my $GroupsFolder = $System->child('Groups');
    foreach my $Alias ( qw( TestGroupX TestGroupY ) ) {
        $GroupsFolder->insertChild('Group', {
            'Alias' => $Alias,
        });
    }

    return 1;
}

sub Cleanup {
    my $System = LoadRootObject();
    my $GroupsFolder = $System->child('Groups');
    foreach my $Alias ( qw( TestGroupX TestGroupY ) ) {
        $GroupsFolder->child($Alias)->delete if $GroupsFolder->existsChild($Alias);
    }

    LoadClassByAlias($CLASSNAME)->delete if ExistsClassByAlias($CLASSNAME);

    return 1;
}

RunScript(
    Message => 'long',
    Sub     => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub   => \&Main,
        );
    },
);