## This test-case is dependent on a DemoShop instance
use Test::More tests => 7;
use strict;

use DE_EPAGES::Core::API::File qw ( DeleteFile );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectExportDriver qw ( ExportObjects );
use DE_EPAGES::Object::API::XMLStructureImportHandler;
use DE_EPAGES::Permission::API::Constants qw ( $ATTRIBUTE_PERMISSION_READ );
use DE_EPAGES::Permission::API::Table::AttributePermission qw ( InsertAttributePermission );
use Readonly;

Readonly::Scalar my $EXPORT_FILENAME => 'Temp.xml';

sub TestMemberOfExport {
    ExportObjects(
        '/Shops/DemoShop',
        $EXPORT_FILENAME,
        { encoding => 'utf-8' },
        [ 'Groups' ],
    );

    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $Struct = $ImportHandler->getXMLStructure($EXPORT_FILENAME);
    my $XMLElement = '/Groups/Merchants/"MemberOf{Group->/Groups/TestSystemMerchants}"';
    ok( $Struct->existsElement($XMLElement),
        'export contains expected memberOf information' );

    return;
}

sub TestAttributePermissionExport {
    ExportObjects(
        '/',
        $EXPORT_FILENAME,
        { encoding => 'utf-8' },
        [ 'Groups' ],
    );

    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $Struct = $ImportHandler->getXMLStructure($EXPORT_FILENAME);
    my $XMLElement = '/Groups/TestSystemMerchants/AttributePermission{' .
        'Class->Shop;Attribute->Slogan;Permission->r}';
    ok( $Struct->existsElement($XMLElement),
        'export contains expected attribute permission' );

    return;
}

sub Main {
    SKIP: {
        my $SkipTests = 7;
        skip('DemoShop is not available on store', $SkipTests )
            unless ExistsObjectByPath('/Shops/DemoShop');

        ok(Init(), 'initialization done');

        TestMemberOfExport();
        TestAttributePermissionExport();

        ok( CleanUp(), 'cleanup done');
    }

    return;
}

sub Init {
    CleanUp();

    # verify that the group 'Merchants' exists in a shop
    my $Shop = LoadObjectByPath('/Shops/DemoShop');
    my $Groups = $Shop->child('Groups');
    ok( $Groups->existsChild('Merchants'), 'group merchants exists' )
        or die 'missing group \'Merchants\' in DemoShop';

    # create new system group 'TestSystemMerchants' and add 'Merchants' as a member
    my $System = LoadRootObject();
    my $TestSystemMerchants = $System->child('Groups')->insertChild('Group', {
        Alias => 'TestSystemMerchants'
    });
    my $Merchants = $Groups->child('Merchants');
    $TestSystemMerchants->addMember($Merchants);

    ok( $Merchants->isMemberOf($TestSystemMerchants),
        'merchants are in group systemmerchants' );
    ok( $TestSystemMerchants->existsMember($Merchants),
        'systemmerchants has member merchants' );

    # insert attribute permission on 'Slogan'
    my $Slogan = $Shop->class->attribute('Slogan');
    InsertAttributePermission({
        'TrusteeID'   => $TestSystemMerchants->id,
        'AttributeID' => $Slogan->id,
        'Permission'  => $ATTRIBUTE_PERMISSION_READ,
    });

    return 1;
}

sub CleanUp {
    DeleteFile($EXPORT_FILENAME);

    my $System = LoadRootObject();
    my $Groups = $System->child('Groups');
    ## deletes also attribute permissions via hook
    $Groups->child('TestSystemMerchants')->delete
        if $Groups->existsChild('TestSystemMerchants');

    return 1;
}

RunScript(
    Message => 'long',
    Sub     => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub   => \&Main,
        );
    },
);

1;