use Test::More tests => 4;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadClassByAlias
    LoadObjectByPath
    LoadRootObject
);
use Test::Deep;

my $ROOTPATH = '/Test';

#========================================================================================
# function     TestScalePrice
# state        private
#----------------------------------------------------------------------------------------
# description  This test validates that scale prices are neither deleted nor inserted
#               into the database as long as they do not change (cf. EPG-28279)
#========================================================================================
sub TestScalePrice {  ## no critic qw(DE_EPAGES::Style::RequireCommentForExportedFunctions)
    my $Shop = LoadObjectByPath($ROOTPATH)->child('TestShop');
    my $Class = LoadClassByAlias('PriceList');
    my $PriceListFolder = $Shop->folder('PriceLists');
    my $PriceList = $Class->insertObject({
        Alias        => 'NewPriceList',
        Parent       => $PriceListFolder,
        CurrencyID   => 'EUR',
    });
    my $Product = $Shop->folder('Products')->child('TestProduct');
    my $TaxModel = 0;
    my $ahScalePrices = [
        {
            Quantity => 1,
            Price    => 10,
        },
        {
            Quantity => 10,
            Price    => 100,
        },
        {
            Quantity => 20,
            Price    => 200,
        },
    ];


    # (1) set initial scale prices and receive their scalepriceids
    $PriceList->setScalePrices($Product, $TaxModel, $ahScalePrices);

    my $dbi = GetCurrentDBHandle();
    my $SQL = <<END_SQL;
    SELECT scalepriceid
    FROM scaleprice
    WHERE productid = ?
END_SQL

    my $aaRows = $dbi->execute( $SQL, [ $Product->id ] );
    my @ScalePriceIDs = map { $_->[0] } @{ $aaRows };


    # (2) set scale prices again, although they didn't change
    #     --> scalepriceids should not change
    $PriceList->setScalePrices($Product, $TaxModel, $ahScalePrices);
    $aaRows = $dbi->execute( $SQL, [ $Product->id ] );
    my @UpdatedScalePriceIDs = map { $_->[0] } @{ $aaRows };
    cmp_deeply(\@UpdatedScalePriceIDs, \@ScalePriceIDs, 'ok');


    # (3) update a single scale price
    #     --> scalepriceids should still not change
    $ahScalePrices->[1]->{Price} = 9;
    $PriceList->setScalePrices($Product, $TaxModel, $ahScalePrices);
    $aaRows = $dbi->execute( $SQL, [ $Product->id ] );
    @UpdatedScalePriceIDs = map { $_->[0] } @{ $aaRows };
    cmp_deeply(\@UpdatedScalePriceIDs, \@ScalePriceIDs, 'ok');


    return;
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);

    return 1;
}

sub Init {
    Cleanup();

    my $Folder = LoadRootObject()->insertChild('Object', { Alias => 'Test' });
    my $Shop = $Folder->insertChild('Shop', { Alias => 'TestShop' });

    $Shop->featureMaxValue('Products', 1);
    $Shop->featureMaxValue('PriceLists', 1);

    my $ProductFolder = $Shop->get('ProductFolder');
    $ProductFolder->insertChild('Product', { Alias => 'TestProduct' });

    return 1;
}

RunScript(
    Message => 'long',
    Sub     => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub   => sub {
                ok( Init(), 'initialization done' );

                TestScalePrice();

                ok( Cleanup(), 'cleanup done' );
            }
        );
    },
);