use Test::More tests => 16;
use strict;

use DateTime;
use DE_EPAGES::Core::API::PerlTools qw ( AddDuration SubtractDuration );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::MailType::API::Object::MailType;
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Product::API::Product qw ( UpdateAvailability );

my $ROOTPATH = '/Test';

#========================================================================================
# function     TestUpdateAvailability
# state        private
#----------------------------------------------------------------------------------------
# description  Test checks if given product availability information---which includes
#               IsAvailable, AvailabilityDate, and ExpiryDate---are updated accordingly
#               when the procedure UpdateAvailability is called.
#========================================================================================
sub TestUpdateAvailability {  ## no critic qw(DE_EPAGES::Style::RequireCommentForExportedFunctions)
    my $Shop = LoadObjectByPath($ROOTPATH)->child('TestShop');
    my $Product = $Shop->folder('Products')->child('TestProduct');

    my $CurrentDate = DateTime->today();
    my $PastDate    = SubtractDuration(DateTime->today(), 'days', 1);
    my $FutureDate  = AddDuration(DateTime->today(), 'days', 1);

    ok(   $PastDate->epoch < $CurrentDate->epoch, 'past date set correctly'   );
    ok( $FutureDate->epoch > $CurrentDate->epoch, 'future date set correctly' );


    #==Availability======================================================================

    $Product->set({
        IsAvailable      => 0,
        AvailabilityDate => undef,
        ExpiryDate       => undef,
    });

    # (1)
    $Product->set({
        IsAvailable      => 0,
        AvailabilityDate => undef,
    });
    _UpdateAvailability();
    is( $Product->get('IsAvailable'), 0,
        'Availability: IsAvailable: 0 (0, undef)' );

    # (2)
    $Product->set({
        IsAvailable      => 0,
        AvailabilityDate => $CurrentDate,
    });
    _UpdateAvailability();
    is( $Product->get('IsAvailable'), 1,
        'Availability: IsAvailable: 1 (0, current)' );

    # (3)
    $Product->set({
        IsAvailable      => 0,
        AvailabilityDate => $PastDate,
    });
    _UpdateAvailability();
    is( $Product->get('IsAvailable'), 1,
        'Availability: IsAvailable: 1 (0, past)' );

    # (4)
    $Product->set({
        IsAvailable      => 0,
        AvailabilityDate => $FutureDate,
    });
    _UpdateAvailability();
    is( $Product->get('IsAvailable'), 0,
        'Availability: IsAvailable: 0 (0, future)' );

    #==Expiry============================================================================

    $Product->set({
        IsAvailable      => 1,
        AvailabilityDate => undef,
        ExpiryDate       => undef,
    });

    # (1)
    $Product->set({
        IsAvailable => 1,
        ExpiryDate  => undef,
    });
    _UpdateAvailability();
    is( $Product->get('IsAvailable'), 1,
        'Expiry: IsAvailable: 1 (1, undef)' );

    # (2)
    $Product->set({
        IsAvailable => 1,
        ExpiryDate  => $CurrentDate,
    });
    _UpdateAvailability();
    is( $Product->get('IsAvailable'), 1,
        'Expiry: IsAvailable: 1 (1, current)' );

    # (3)
    $Product->set({
        IsAvailable => 1,
        ExpiryDate  => $PastDate,
    });
    _UpdateAvailability();
    is( $Product->get('IsAvailable'), 0,
        'Expiry: IsAvailable: 0 (1, past)' );

    # (4)
    $Product->set({
        IsAvailable => 1,
        ExpiryDate  => $FutureDate,
    });
    _UpdateAvailability();
    is( $Product->get('IsAvailable'), 1,
        'Expiry: IsAvailable: 1 (1, future)' );

    #==Combinations======================================================================

    # (1)
    $Product->set({
        IsAvailable      => 0,
        AvailabilityDate => $CurrentDate,
        ExpiryDate       => $FutureDate,
    });
    _UpdateAvailability();
    is( $Product->get('IsAvailable'), 1,
        'Combination: IsAvailable: 1 (0, current, future)' );

    # (2)
    $Product->set({
        IsAvailable      => 0,
        AvailabilityDate => $FutureDate,
        ExpiryDate       => $FutureDate,
    });
    _UpdateAvailability();
    is( $Product->get('IsAvailable'), 0,
        'Combination: IsAvailable: 0 (0, future, future)' );

    # (3)
    $Product->set({
        IsAvailable      => 1,
        AvailabilityDate => $CurrentDate,
        ExpiryDate       => $FutureDate,
    });
    _UpdateAvailability();
    is( $Product->get('IsAvailable'), 1,
        'Combination (Expiry): IsAvailable: 1 (0, current, future)' );

    # (4)
    $Product->set({
        IsAvailable      => 1,
        AvailabilityDate => $PastDate,
        ExpiryDate       => $FutureDate,
    });
    _UpdateAvailability();
    is( $Product->get('IsAvailable'), 1,
        'Combination (Expiry): IsAvailable: 1 (0, past, future)' );

    return;
}

sub _UpdateAvailability {
    {
        local *DE_EPAGES::MailType::API::Object::MailType::send = sub {};
        UpdateAvailability();
    }

    return;
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);

    return 1;
}

sub Init {
    Cleanup();

    my $Folder = LoadRootObject()->insertChild('Object', { Alias => 'Test' });
    my $Shop = $Folder->insertChild('Shop', { Alias => 'TestShop' });

    $Shop->featureMaxValue('Products', 1);
    my $ProductFolder = $Shop->get('ProductFolder');
    $ProductFolder->insertChild('Product', { Alias => 'TestProduct' });

    return 1;
}

RunScript(
    Message => 'long',
    Sub     => sub {
        RunOnStore(
            Store => GetTestSite(),
            Sub   => sub {
                ok( Init(), 'initialization done' );

                TestUpdateAvailability();

                ok( Cleanup(), 'cleanup done' );
            }
        );
    },
);