use Test::More tests => 14;
use warnings;
use strict;
use utf8;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::PerlTools qw( SubtractDuration);
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetTestSite );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath LoadRootObject ExistsObjectByPath);
use DE_EPAGES::Presentation::API::Session qw( GetSessionAttribute InsertSession SetSessionAttribute CreateAdminSFSessionID TestAdminSFSession);
use DE_EPAGES::Presentation::API::Table::Session qw( GetInfoSessionBySessionID DeleteSession ExistsSession);

my $ROOTPATH = '/Test';

sub Main {

    Init();

    # fake
    local *DE_EPAGES::Presentation::API::Session::GetAppServer = sub {
        return Mock::AppServer->new;
    };

    my $Shop = LoadObjectByPath($ROOTPATH)->child('TestShop1');

    ## no critic qw( ProhibitMagicNumbers )
    # create session for admin user
    my $AdminUser = $Shop->folder('Users')->child('admin');
    my $AdminSessionID = InsertSession( $AdminUser, 1 );
    my $hAdminSession = GetInfoSessionBySessionID( $AdminSessionID );
    is( length $AdminSessionID, 32, 'AdminSessionID length' );

    my $CryptAdminSessionID = CreateAdminSFSessionID($Shop,$AdminSessionID);
    ok( defined $CryptAdminSessionID, 'Crypted admin session id is defined' );
    is( length $CryptAdminSessionID, 64, 'CryptAdminSessionID length' );

    # not closed by merchant
    $Shop->set({'ClosedByMerchant' => 0});
    ok( 0 == TestAdminSFSession($Shop,undef), 'Site not closed by merchant' );
    $Shop->set({'ClosedByMerchant' => 1});

    # no cookie defined
    ok( -1 == TestAdminSFSession($Shop,undef), 'No cookie informations available' );

    # wrong cookie session
    ok( -1 == TestAdminSFSession($Shop,'08154711'), 'Wrong cookie information used' );

    # correct cookie session
    ok( 1 == TestAdminSFSession($Shop,$CryptAdminSessionID), 'Correct cookie information used' );

    # user is not a merchant
    my $NoMerchantUser = $Shop->folder('Users')->child('NoMerchant');
    my $NoMerchantSessionID = InsertSession( $NoMerchantUser, 1 );
    my $NoMerchantCryptAdminSessionID = CreateAdminSFSessionID($NoMerchantUser->getSite,$NoMerchantSessionID);
    ok( defined $NoMerchantCryptAdminSessionID, 'Crypted no merchant session id is defined' );
    ok( -1 == TestAdminSFSession($NoMerchantUser->getSite,$NoMerchantCryptAdminSessionID), 'User is not a merchant' );

    # session is closed
    SetSessionAttribute( $hAdminSession->{'IntSessionID'}, 'IsClosed', 1);
    ok( -1 == TestAdminSFSession($Shop,$CryptAdminSessionID), 'Session is closed' );
    SetSessionAttribute( $hAdminSession->{'IntSessionID'}, 'IsClosed', 0);

    #test session timeout
    my $CreationDate = GetSessionAttribute($hAdminSession->{'IntSessionID'}, 'CreationDate');
    my $SessionTimeout = LoadRootObject()->get('SessionTimeout');

    # no timeout (1 minute left)
    SubtractDuration( $CreationDate, 'minutes', $SessionTimeout -1 ) ;
    SetSessionAttribute( $hAdminSession->{'IntSessionID'}, 'CreationDate',$CreationDate );
    ok( 1 == TestAdminSFSession($Shop,$CryptAdminSessionID), 'Session not timed out' );
    # timeout
    SubtractDuration( $CreationDate, 'minutes', 1 ) ;
    SetSessionAttribute( $hAdminSession->{'IntSessionID'}, 'CreationDate',$CreationDate );
    ok( -1 == TestAdminSFSession($Shop,$CryptAdminSessionID), 'Session timed out' );

    # session is not existing
    DeleteSession($hAdminSession->{'IntSessionID'});
    ok(!ExistsSession($hAdminSession->{'IntSessionID'}), 'Delete session.');
    ok( -1 == TestAdminSFSession($Shop,$CryptAdminSessionID), 'Session is not existing' );

    Cleanup();
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $Folder = LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    my $Shop = $Folder->insertChild( 'Shop', { Alias => 'TestShop1' } );
    $Shop->folder('Users')->insertChild( 'User', { Alias => 'NoMerchant' } );
    return 1;
}


## no critic qw( RequireCommentForExportedFunctions )
package Mock::AppServer;

sub new {
    my $class = shift;
    return bless {}, $class;
}

sub address {
    return '0.0.0.0';
}

sub port {
    return 0;
}

sub serverVariables {
    return {
        HTTP_REFERER => 'x',
        REMOTE_ADDR => 'ip',
        REMOTE_HOST => 'host',
        REMOTE_USER => 'user',
    };
}
